﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.IO;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Windows.Storage;
using Windows.System.Display;
using System.Windows.Threading;

namespace WinPhoneFtp.DemoApp
{
    public partial class FileDownload : PhoneApplicationPage
    {
        public FileDownload()
        {
            InitializeComponent();
        }

        String FileToDownload = String.Empty;
        DispatcherTimer timer = null;
        UInt32 downloadedChunckSize = 0;
        UInt32 totalDownloadedSize = 0;

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {

            if (this.NavigationContext.QueryString.Count.Equals(1) && this.NavigationContext.QueryString.ContainsKey("File"))
            {
                FileToDownload = this.NavigationContext.QueryString["File"];
                if ((App.Current as App).ServerCurrentWorkingDirectory.Equals("/"))
                {
                    txtblkDownloadFilename.Text = String.Format("File to download: /{0}", FileToDownload);
                }
                else
                {
                    txtblkDownloadFilename.Text = String.Format("File to download: {0}/{1}", (App.Current as App).ServerCurrentWorkingDirectory, FileToDownload);
                }
            }
            else
            {
                MessageBox.Show("Invalid Page Navigation detected", "FTP Demo App", MessageBoxButton.OK);
                this.NavigationService.Navigate(new Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative));
            }
            base.OnNavigatedTo(e);
        }


        protected override void OnNavigatingFrom(NavigatingCancelEventArgs e)
        {
            (App.Current as App).Client.FtpFileDownloadFailed -= Client_FtpFileDownloadFailed;
            (App.Current as App).Client.FtpFileDownloadSucceeded -= Client_FtpFileDownloadSucceeded;
            (App.Current as App).Client.FtpFileTransferProgressed -= Client_FtpFileTransferProgressed;
            base.OnNavigatingFrom(e);
        }

        private async void btnDownloadFile_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            btnDownloadFile.IsEnabled = false;
            SystemTray.SetProgressIndicator(this, new ProgressIndicator() { IsIndeterminate = true, Text = "Downloading file over FTP", IsVisible = true });
            timer = new DispatcherTimer() { Interval = TimeSpan.FromSeconds(1) };
            timer.Tick += timer_Tick;
            (App.Current as App).Client.FtpFileDownloadFailed += Client_FtpFileDownloadFailed;
            (App.Current as App).Client.FtpFileDownloadSucceeded += Client_FtpFileDownloadSucceeded;
            (App.Current as App).Client.FtpFileTransferProgressed += Client_FtpFileTransferProgressed;
            StorageFile localFile = await Windows.Storage.ApplicationData.Current.LocalFolder.CreateFileAsync(FileToDownload, CreationCollisionOption.ReplaceExisting);
            await (App.Current as App).Client.DownloadFileAsync(await localFile.OpenStreamForWriteAsync(), FileToDownload);
            timer.Start();
        }

        void timer_Tick(object sender, EventArgs e)
        {
            this.downloadedChunckSize = 0;
            if (SystemTray.ProgressIndicator != null)
            {
                SystemTray.ProgressIndicator.Text = String.Format("{0} KB downloaded", Convert.ToSingle(totalDownloadedSize / 1024).ToString("F2"));
            }
        }

        void Client_FtpFileTransferProgressed(object sender, FtpService.FtpFileTransferProgressedEventArgs e)
        {
            this.downloadedChunckSize += e.BytesTransferred;
            this.totalDownloadedSize += e.BytesTransferred;
        }

        void Client_FtpFileDownloadSucceeded(object sender, FtpService.FtpFileTransferEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                btnDownloadFile.IsEnabled = true;
                SystemTray.ProgressIndicator = null;
                timer.Stop();
                MessageBox.Show("File downloaded successfully", "FTP Demo App", MessageBoxButton.OK);
                this.NavigationService.Navigate(new Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative));
            });
        }

        void Client_FtpFileDownloadFailed(object sender, FtpService.FtpFileTransferFailedEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                btnDownloadFile.IsEnabled = true;
                SystemTray.ProgressIndicator = null;
                timer.Stop();
                MessageBox.Show(String.Format("Unable to download file. Reason: {0}", e.FileTransferFailReason), "FTP Demo App", MessageBoxButton.OK);
                this.NavigationService.Navigate(new Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative));
            });
        }

        private void appbarmnuDisconnect_Click(object sender, EventArgs e)
        {
            this.NavigationService.Navigate(new Uri("/Disconnect.xaml", UriKind.Relative));
        }
    }
}