﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using System.Windows.Threading;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Windows.Storage;

namespace WinPhoneFtp.DemoApp
{
    public partial class FileUpload : PhoneApplicationPage
    {
        public FileUpload()
        {
            InitializeComponent();
        }

        String uploadFilename = String.Empty;
        DispatcherTimer timer = null;
        UInt32 uploadedChunckSize = 0;
        UInt32 totalUploadedSize = 0;

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            if (this.NavigationContext.QueryString.Count.Equals(1) && this.NavigationContext.QueryString.ContainsKey("File"))
            {
                uploadFilename = this.NavigationContext.QueryString["File"];
                txtblkUploadFilename.Text = String.Format("File to upload: {0}", uploadFilename);
            }
            else
            {
                MessageBox.Show("Invalid Page Navigation detected", "FTP Demo App", MessageBoxButton.OK);
                this.NavigationService.Navigate(new Uri("/PhoneDirectoryBrowser.xaml", UriKind.Relative));
            }
            base.OnNavigatedTo(e);
        }

        private async void btnUploadFile_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            btnUploadFile.IsEnabled = false;
            SystemTray.SetProgressIndicator(this, new ProgressIndicator() { IsIndeterminate = true, Text = "Uploading file over FTP", IsVisible = true });
            timer = new DispatcherTimer() { Interval = TimeSpan.FromSeconds(1) };
            timer.Tick += timer_Tick;
            (App.Current as App).Client.FtpFileUploadSucceeded += Client_FtpFileUploadSucceeded;
            (App.Current as App).Client.FtpFileUploadFailed += Client_FtpFileUploadFailed;
            (App.Current as App).Client.FtpFileTransferProgressed += Client_FtpFileTransferProgressed;
            StorageFile localFile = await (App.Current as App).PhoneCurrentWorkingDirectory.Peek().GetFileAsync(uploadFilename);
            await (App.Current as App).Client.UploadFileAsync(await localFile.OpenStreamForReadAsync(), uploadFilename);
            timer.Start();
        }

        void Client_FtpFileTransferProgressed(object sender, FtpService.FtpFileTransferProgressedEventArgs e)
        {
            this.uploadedChunckSize += e.BytesTransferred;
            this.totalUploadedSize += e.BytesTransferred;
        }

        void Client_FtpFileUploadFailed(object sender, FtpService.FtpFileTransferFailedEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                btnUploadFile.IsEnabled = true;
                SystemTray.ProgressIndicator = null;
                timer.Stop();
                MessageBox.Show(String.Format("Unable to upload file. Reason: {0}", e.FileTransferFailReason), "FTP Demo App", MessageBoxButton.OK);
                this.NavigationService.Navigate(new Uri("/PhoneDirectoryBrowser.xaml", UriKind.Relative));
            });
        }

        void Client_FtpFileUploadSucceeded(object sender, FtpService.FtpFileTransferEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                btnUploadFile.IsEnabled = true;
                SystemTray.ProgressIndicator = null;
                timer.Stop();
                MessageBox.Show("File uploaded successfully", "FTP Demo App", MessageBoxButton.OK);
                this.NavigationService.Navigate(new Uri("/PhoneDirectoryBrowser.xaml", UriKind.Relative));
            });
        }

        void timer_Tick(object sender, EventArgs e)
        {
            this.uploadedChunckSize = 0;
            if (SystemTray.ProgressIndicator != null)
            {
                SystemTray.ProgressIndicator.Text = String.Format("{0} KB uploaded", Convert.ToSingle(totalUploadedSize / 1024).ToString("F2"));
            }
        }

        private void appbarmnuDisconnect_Click(object sender, EventArgs e)
        {
            this.NavigationService.Navigate(new Uri("/Disconnect.xaml", UriKind.Relative));
        }
    }
}