﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace WinPhoneFtp.DemoApp
{
    public partial class FtpDirectoryBrowser : PhoneApplicationPage
    {
        public FtpDirectoryBrowser()
        {
            InitializeComponent();
        }

        protected override void OnNavigatingFrom(NavigatingCancelEventArgs e)
        {
            (App.Current as App).Client.FtpDirectoryListed -= Client_FtpDirectoryListed;
        }

        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            while (this.NavigationService.CanGoBack)
            {
                this.NavigationService.RemoveBackEntry();
            }

            (App.Current as App).Client.FtpDirectoryListed += Client_FtpDirectoryListed;
            await (App.Current as App).Client.GetDirectoryListingAsync();
            base.OnNavigatedTo(e);
        }

        protected async override void OnBackKeyPress(System.ComponentModel.CancelEventArgs e)
        {
            switch (MessageBox.Show("Do you want to disconnect and exit", "FTP Demo App", MessageBoxButton.OKCancel))
            {
                case MessageBoxResult.OK:
                    await (App.Current as App).Client.DisconnectAsync();
                    break;

                case MessageBoxResult.Cancel:
                    e.Cancel = true;
                    break;
            }

            base.OnBackKeyPress(e);
        }

        void Client_FtpDirectoryListed(object sender, FtpService.FtpDirectoryListedEventArgs e)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                if (e.GetFilenames().Length > 0)
                {
                    txtblkNoFiles.Visibility = System.Windows.Visibility.Collapsed;
                    lstFiles.Visibility = System.Windows.Visibility.Visible;
                    lstFiles.ItemsSource = e.GetFilenames();
                }
                else
                {
                    txtblkNoFiles.Visibility = System.Windows.Visibility.Visible;
                    lstFiles.Visibility = System.Windows.Visibility.Collapsed;
                }

                List<String> dirList = new List<String>();
                if (!(App.Current as App).ServerCurrentWorkingDirectory.Equals("/"))
                {
                    dirList.Add("[One Folder Up]");
                }
                dirList.AddRange(e.GetDirectories());
                lstDirectories.ItemsSource = dirList;
            });
        }

        private void appbarmnuDisconnect_Click(object sender, EventArgs e)
        {
            this.NavigationService.Navigate(new Uri("/Disconnect.xaml", UriKind.Relative));
        }

        private async void txtblkDirName_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            if ((sender as TextBlock).DataContext.ToString().Equals("[One Folder Up]"))
            {
                await (App.Current as App).Client.ChangeWorkingDirectoryAsync("..");
            }
            else
            {
                await (App.Current as App).Client.ChangeWorkingDirectoryAsync((sender as TextBlock).DataContext.ToString());
            }
            await (App.Current as App).Client.GetDirectoryListingAsync();
        }

        private void txtblkFileName_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            this.NavigationService.Navigate(new Uri(String.Format("/FileDownload.xaml?File={0}", (sender as TextBlock).DataContext.ToString()), UriKind.Relative));
        }

        private void appbarbtnUploadFile_Click(object sender, EventArgs e)
        {
            this.NavigationService.Navigate(new Uri("/PhoneDirectoryBrowser.xaml", UriKind.Relative));
        }
    }
}