﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Windows.Storage;

namespace WinPhoneFtp.DemoApp
{
    public partial class PhoneDirectoryBrowser : PhoneApplicationPage
    {
        public PhoneDirectoryBrowser()
        {
            InitializeComponent();
        }

        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            await RefreshFileSystemListAsync((App.Current as App).PhoneCurrentWorkingDirectory.Peek());
            base.OnNavigatedTo(e);
        }

        private async Task RefreshFileSystemListAsync(StorageFolder currentFolder)
        {
            List<String> files = (from StorageFile file in await currentFolder.GetFilesAsync()
                                  select file.Name).ToList();

            if (files.Count.Equals(0))
            {
                lstFiles.Visibility = System.Windows.Visibility.Collapsed;
                txtblkNoFiles.Visibility = System.Windows.Visibility.Visible;
                lstFiles.ItemsSource = null;
            }
            else
            {
                lstFiles.Visibility = System.Windows.Visibility.Visible;
                txtblkNoFiles.Visibility = System.Windows.Visibility.Collapsed;
                lstFiles.ItemsSource = files;
            }

            List<String> dirList = (from StorageFolder directory in await currentFolder.GetFoldersAsync()
                                    select directory.Name).ToList();
            if (!Windows.Storage.ApplicationData.Current.LocalFolder.Name.Equals(currentFolder.Name))
            {
                dirList.Insert(0, "[One Folder Up]");
            }

            if (dirList.Count.Equals(0))
            {
                lstDirectories.Visibility = System.Windows.Visibility.Collapsed;
                txtblkNoDirectories.Visibility = System.Windows.Visibility.Visible;
                lstDirectories.ItemsSource = null;
            }
            else
            {
                lstDirectories.Visibility = System.Windows.Visibility.Visible;
                txtblkNoDirectories.Visibility = System.Windows.Visibility.Collapsed;
                lstDirectories.ItemsSource = dirList;
            }
        }

        private async void txtblkDirName_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            String dirName = (sender as TextBlock).DataContext.ToString();
            if (dirName.Equals("[One Folder Up]"))
            {
                (App.Current as App).PhoneCurrentWorkingDirectory.Pop();
            }
            else
            {
                (App.Current as App).PhoneCurrentWorkingDirectory.Push(await (App.Current as App).PhoneCurrentWorkingDirectory.Peek().GetFolderAsync((sender as TextBlock).DataContext.ToString()));
            }
            await RefreshFileSystemListAsync((App.Current as App).PhoneCurrentWorkingDirectory.Peek());
        }

        private void txtblkFileName_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            this.NavigationService.Navigate(new Uri(String.Format("/FileUpload.xaml?File={0}", (sender as TextBlock).DataContext.ToString()), UriKind.Relative));
        }
    }
}